@@ -1,5 +1,5 @@
 #' Parameter Estimation for Cognitive Diagnosis Models (CDMs) by MMLE/EM or MMLE/BM Algorithm.
-#'
+#' 
 #' @description
 #' A function to estimate parameters for cognitive diagnosis models by MMLE/EM (de la Torre, 2009; de la Torre, 2011)
 #' or MMLE/BM (Ma & Jiang, 2020) algorithm. The function imports various functions from the \code{GDINA} package,
@@ -19,8 +19,11 @@
 #'            \code{"LLM"}, or \code{"rRUM"}. Default = \code{"GDINA"}.
 #' @param method  Type of method to estimate CDMs' parameters; one out of \code{"EM"} and \code{"BM"}. Default = \code{"EM"}. 
 #'                However, \code{"BM"} is only available when \code{method = "GDINA"}.
+#' @param att.str Specify attribute structures. \code{NULL}, by default, means there is no structure. Attribute structure 
+#'                needs be specified as a list - which will be internally handled by \code{\link[GDINA]{att.structure}} function. 
+#'                See examples. It can also be a matrix giving all permissible attribute profiles.
 #' @param mono.constraint Logical indicating whether monotonicity constraints should be fulfilled in estimation.
-#'                        Default = \code{TRUE}.
+#'                        Default = \code{FALSE}.
 #' @param maxitr A vector for each item or nonzero category, or a scalar which will be used for all items
 #'                 to specify the maximum number of EM or BM cycles allowed. Default = \code{2000}.
 #' @param verbose Can be \code{0}, \code{1} or \code{2}, indicating to print no information, information
@@ -134,17 +137,17 @@
 #' @return
 #' An object of class \code{CDM} containing the following components:
 #' \describe{
-#'  \item{analysis.obj}{An \code{\link[GDINA]{GDINA}} object gained from \code{GDINA} package or an 
-#'                      \code{list} after BM algorithm, depending on which estimation is used.}
-#'  \item{alpha}{Individuals' attribute parameters calculated by EAP method}
-#'  \item{P.alpha.Xi}{Individual's posterior probability}
-#'  \item{alpha.P}{Individuals' marginal mastery probabilities matrix}
-#'  \item{P.alpha}{Attribute prior weights for calculating marginalized likelihood in the last iteration}
-#'  \item{model.fit}{Some basic model-fit indices, including \code{Deviance}, \code{npar}, \code{AIC}, \code{BIC}. @seealso \code{\link[Qval]{fit}}}
-#'  \item{arguments}{A list containing all input arguments}
+#'  \item{\code{analysis.obj}}{An \code{\link[GDINA]{GDINA}} object gained from \code{GDINA} package or an 
+#'                             \code{list} after BM algorithm, depending on which estimation is used.}
+#'  \item{\code{alpha}}{Individuals' attribute parameters calculated by EAP method}
+#'  \item{\code{P.alpha.Xi}}{Individual's posterior probability}
+#'  \item{\code{alpha.P}}{Individuals' marginal mastery probabilities matrix}
+#'  \item{\code{P.alpha}}{Attribute prior weights for calculating marginalized likelihood in the last iteration}
+#'  \item{\code{model.fit}}{Some basic model-fit indices, including \code{Deviance}, \code{npar}, \code{AIC}, \code{BIC}. @seealso \code{\link[Qval]{fit}}}
+#'  \item{\code{pattern}}{The attribute mastery pattern matrix containing all possible attribute mastery pattern.}
+#'  \item{\code{arguments}}{A list containing all input arguments}
 #' }
 #' 
-#'
 #' @author Haijiang Qin <Haijiang133@outlook.com>
 #'
 #' @references
@@ -190,7 +193,7 @@
 #'   P1 = runif(I, 0.8, 1.0)
 #' )
 #' data.obj <- sim.data(Q = Q, N = 500, IQ = IQ,
-#'                          model = "GDINA", distribute = "horder")
+#'                          model = "GDINA", distribute = "mvnorm")
 #'
 #'
 #' ## using MMLE/EM to fit GDINA model
@@ -226,7 +229,7 @@
 #'
 #'################################################################
 #'#                           Example 3                          #
-#'#              fit using MMLE/EM to fit the ACDM               #
+#'# fit using MMLE/EM to fit the GDINA with attribute structures #
 #'################################################################
 #' \donttest{
 #' set.seed(123)
@@ -234,19 +237,23 @@
 #' library(Qval)
 #'
 #' ## generate Q-matrix and data to fit
-#' K <- 5
+#' K <- 3
 #' I <- 30
-#' Q <- sim.Q(K, I)
+#' att.str <- list(c(1,2),
+#'                 c(2,3))
+#' Q <- sim.Q(K, I, att.str)
 #' IQ <- list(
 #'   P0 = runif(I, 0.0, 0.2),
 #'   P1 = runif(I, 0.8, 1.0)
 #' )
-#' data.obj <- sim.data(Q = Q, N = 500, IQ = IQ,
-#'                  model = "ACDM", distribute = "horder")
-#'
-#' ## using MMLE/EM to fit GDINA model
-#' CDM.obj <- CDM(data.obj$dat, Q, model = "ACDM",
-#'                method = "EM", maxitr = 2000, verbose = 1)
+#' data.obj <- sim.data(Q = Q, N = 500, IQ = IQ, att.str=att.str, 
+#'                      model = "GDINA")
+#' 
+#' ## using MMLE/EM to fit GDINA model with attribute structures
+#' CDM.obj <- CDM(data.obj$dat, Q, model = "GDINA", att.str=att.str,
+#'                 method = "EM", maxitr = 2000, verbose = 1)
+#' 
+#' summary(CDM.obj)
 #'}
 #'
 #' @export
@@ -255,15 +262,20 @@
 #' @importFrom plyr llply
 #' @importFrom nloptr slsqp
 #'
-
-CDM <- function(Y, Q, model="GDINA", method="EM",
-                mono.constraint=TRUE, maxitr=2000, verbose=1){
-
+CDM <- function(Y, Q, model="GDINA", method="EM", att.str=NULL, 
+                mono.constraint=FALSE, maxitr=2000, verbose=1){
+  
   if (!method %in% c("EM", "BM"))
     stop("method must be 'EM' or 'BM'.\n")
-
+  
   if(method == "BM" & model != "GDINA")
-    stop("model must be 'GDINA' when 'BM' !!! \n")
+    stop("model must be 'GDINA' when 'BM' ! \n")
+  
+  if(!is.null(att.str) & all(!class(att.str) %in% c("list", "matrix"))){
+    stop("att.str must be NULL, a list or a matrix ! \n")
+  }else if(!is.null(att.str) & (!model %in% c("GDINA", "DINA", "DINO") | method != "EM") ){
+    stop("model must be one of 'GDINA', 'DINA' and 'DINO' and method must be 'EM' when att.str is not NULL !\n")
+  }
   
   CDMcall <- match.call()
   
@@ -272,26 +284,32 @@ CDM <- function(Y, Q, model="GDINA", method="EM",
   N <- nrow(Y)
   I <- nrow(Q)
   K <- ncol(Q)
-  KS <- attributepattern(K)
-  L <- nrow(KS)
+  if(is.null(att.str)){
+    pattern <- attributepattern(K)
+  }else if(any(class(att.str) %in% "list")){
+    pattern <- att.structure(att.str, K)$att.str
+  }else{
+    pattern <- att.str
+  }
+  L <- nrow(pattern)
 
   if(method == "EM"){
     if(model=="LCDM"){
       CDM.obj <- GDINA(Y, Q, verbose = verbose, mono.constraint = mono.constraint, control=list(maxitr = maxitr),
-                       model = "GDINA",linkfunc="logit")
+                       att.str=att.str, model = "GDINA",linkfunc="logit")
       if(verbose != 0)
         cat("\n")
     }else{
-      CDM.obj <- GDINA(Y, Q, model = model, verbose = verbose, mono.constraint = mono.constraint, control=list(maxitr = maxitr))
+      CDM.obj <- GDINA(Y, Q, model = model, verbose = verbose, att.str=att.str, 
+                       mono.constraint = mono.constraint, control=list(maxitr = maxitr))
       if(verbose != 0)
         cat("\n")
     }
-    CDM.obj$Y <- Y
-    CDM.obj$Q <- Q
+
     P.alpha.Xi <- exp(indlogPost(CDM.obj))
     P.alpha <- colSums(P.alpha.Xi) / sum(P.alpha.Xi)
     alpha.P <- personparm(CDM.obj, what = "mp")
-    alpha <- KS[apply(P.alpha.Xi, 1, which.max), ]
+    alpha <- pattern[apply(P.alpha.Xi, 1, which.max), ]
   }
   if(method == "BM"){
     oopts <- options('nloptr.show.inequality.warning' = FALSE)
@@ -416,8 +434,8 @@ CDM <- function(Y, Q, model="GDINA", method="EM",
     CDM.obj$testfit$BIC <- CDM.obj$testfit$Deviance + log(N)*CDM.obj$testfit$npar
     P.alpha.Xi <- exp(estep$logpost)
     P.alpha <- colSums(P.alpha.Xi) / sum(P.alpha.Xi)
-    alpha.P <- t(apply(P.alpha.Xi, 1, function(x)(return(colSums(x*KS)))))
-    alpha <- KS[apply(P.alpha.Xi, 1, which.max), ]
+    alpha.P <- t(apply(P.alpha.Xi, 1, function(x)(return(colSums(x*pattern)))))
+    alpha <- pattern[apply(P.alpha.Xi, 1, which.max), ]
     if(verbose != 0)
       cat("\n")
   
@@ -430,11 +448,11 @@ CDM <- function(Y, Q, model="GDINA", method="EM",
                 BIC=CDM.obj$testfit$BIC)
   
   res <- list(analysis.obj=CDM.obj, alpha=alpha, P.alpha.Xi=P.alpha.Xi,
-              alpha.P=alpha.P, P.alpha=P.alpha,
+              alpha.P=alpha.P, P.alpha=P.alpha, pattern=pattern, 
               model.fit=modelfit, 
               call = CDMcall, 
               arguments = list(
-                Y=Y, Q=Q, model=model, method=method,
+                Y=Y, Q=Q, model=model, method=method, att.str=att.str, 
                 mono.constraint=mono.constraint, maxitr=maxitr, verbose=verbose
               ))
 
