@@ -7,7 +7,7 @@
 #' 2021), and MLR-B (Tu et al., 2022). It supports different iteration methods (test
 #' level or item level; Najera et al., 2020; Najera et al., 2021; Tu et al., 2022) and
 #' can apply various attribute search methods (ESA, SSA, PAA; de la Torre, 2008; Terzi, &
-#' de la Torre, 2018).
+#' de la Torre, 2018; Qin & Guo, 2025).
 #'
 #' @section The GDI method:
 #' The GDI method (de la Torre & Chiu, 2016), as the first Q-matrix validation method
@@ -42,7 +42,8 @@
 #' the maximum discrimination. They selected an appropriate \eqn{PVAF} cut-off point to achieve
 #' a balance between q-vector fit and parsimony. According to previous studies,
 #' the \eqn{PVAF} cut-off point is typically set at 0.95 (Ma & de la Torre, 2020; Najera et al., 2021).
-#' Najera et al. (2019) proposed using multinomial logistic regression to predict a more appropriate cut-off point for \eqn{PVAF}. 
+#' Najera et al. (2019; 2020) proposed using multinomial logistic regression to predict a more 
+#' appropriate cut-off point for \eqn{PVAF}. 
 #' The cut-off point is denoted as \eqn{eps}, and the predicted regression equation is as follows:
 #' 
 #' \deqn{ 
@@ -50,9 +51,9 @@
 #'    = \text{logit}(eps) 
 #'    = -0.405 + 2.867 \cdot IQ + 4.840 \times 10^{-4} \cdot N - 3.316 \times 10^{-3} \cdot I
 #'  }
-#'  Where \eqn{IQ} represents the question quality, calculated as the negative difference between the probability of an examinee 
-#'  with all attributes answering the question correctly and the probability of an examinee with no attributes answering the question correctly 
-#'  (\eqn{IQ = P\left( \boldsymbol{1} \right) - P\left( \boldsymbol{0} \right)}), 
+#'  Where \eqn{IQ} represents the item quality, calculated as the average difference between the probability of examinee 
+#'  with all attributes answering the item correctly and the probability of an examinee with no attributes answering the question correctly 
+#'  (\eqn{IQ = \frac{1}{I} \sum_{i=1}^{I} \left[ P_{i}\left( \boldsymbol{1} \right) - P_{i}\left( \boldsymbol{0} \right)\right]}), 
 #'  and \eqn{N} and \eqn{I} represent the number of examinees and the number of questions, respectively.
 #' 
 #' 
@@ -84,7 +85,9 @@
 #' 
 #' \eqn{\boldsymbol{M}_i} is a \eqn{2^{K^\ast} \times 2^{K^\ast}} matrix (@seealso \code{\link[Qval]{get.Mmatrix}}) 
 #' that represents the relationship between the parameters of item \eqn{i} and the attribute mastery patterns. The 
-#' rows represent different mastery patterns, while the columns represent different item parameters.
+#' rows represent different mastery patterns, while the columns represent different item parameters. 
+#' 
+#' In Package Qval, \link[parallel]{parLapply} will be used to accelerate the Wald method.
 #'
 #' @section The Hull method:
 #' The Hull method (Najera et al., 2021) addresses the issue of the cut-off point in the GDI
@@ -118,7 +121,9 @@
 #' that exclude any insignificant regression coefficients, it selects the q-vector corresponding to
 #' the equation with the minimum \eqn{AIC} value as the validation result. The performance of this method in both the
 #' LCDM and GDM models even surpasses that of the Hull method (Tu et al., 2022), making it an efficient and reliable
-#' approach for Q-matrix validation.
+#' approach for Q-matrix validation. 
+#' 
+#' In Package Qval, \link[parallel]{parLapply} will be used to accelerate the MLR-B method when \code{search.method} is not \code{"PAA"}.
 #' 
 #' @section The \eqn{\beta} method:
 #' The \eqn{\beta} method (Li & Chen, 2024) addresses the Q-matrix validation problem from the 
@@ -158,7 +163,7 @@
 #' 
 #' Please note that the \eqn{\beta} method has different meanings when applying different search algorithms. 
 #' For more details, see section 'Search algorithm' below.
-#'  
+#' 
 #' @section Iterative procedure:
 #' The iterative procedure that one item modification at a time is item level iteration (\code{ iter.level = "item"}) in (Najera
 #' et al., 2020, 2021). The steps of the \code{item} level iterative procedure algorithm are as follows:
@@ -320,7 +325,7 @@
 #' @param par.method  Type of method to estimate CDMs' parameters; one out of \code{"EM"}, \code{"BM"}. Default = \code{"EM"}. 
 #'                However, \code{"BM"} is only available when \code{method = "GDINA"}.
 #' @param mono.constraint Logical indicating whether monotonicity constraints should be fulfilled in estimation.
-#'                        Default = \code{TRUE}.
+#'                        Default = \code{FALSE}.
 #' @param model Type of model to fit; can be \code{"GDINA"}, \code{"LCDM"}, \code{"DINA"}, \code{"DINO"}
 #'              , \code{"ACDM"}, \code{"LLM"}, or \code{"rRUM"}. Default = \code{"GDINA"}.
 #'              @seealso \code{\link[Qval]{CDM}}.
@@ -331,12 +336,12 @@
 #'               Default = \code{"GDI"}. See details.
 #' @param search.method Character string specifying the search method to use during validation.
 #'   \describe{
-#'     \item{"ESA"}{for exhaustive search algorithm. Cannot be used with the \code{"Wald"} method.}
-#'     \item{"SSA"}{for sequential search algorithm (see de la Torre, 2008; Terzi & de la Torre, 2018). 
-#'                  It will be equal to \code{"forward"} when \code{method = "Wald"}.}
-#'     \item{"PAA"}{for priority attribute algorithm. }
-#'     \item{"stepwise"}{only for the \code{"Wald" method}}
-#'     \item{"beta"}{only for the \code{"beta" method}}
+#'     \item{\code{"ESA"}}{for exhaustive search algorithm. Cannot be used with the \code{"Wald"} method.}
+#'     \item{\code{"SSA"}}{for sequential search algorithm (see de la Torre, 2008; Terzi & de la Torre, 2018). 
+#'                         It will be equal to \code{"forward"} when \code{method = "Wald"}.}
+#'     \item{\code{"PAA"}}{for priority attribute algorithm. }
+#'     \item{\code{"stepwise"}}{only for the \code{"Wald"} method}
+#'     \item{\code{"beta"}}{only for the \code{"beta"} method}
 #'   }
 #' @param iter.level Can be \code{"no"}, \code{"item"} level, \code{"test.att"} or \code{"test"} level. Default = \code{"no"} and 
 #'                   \code{"test.att"} can not for \code{"Wald"} and \code{"MLR-B"}. See details.
@@ -354,19 +359,20 @@
 #' @return
 #' An object of class \code{validation} containing the following components:
 #' \describe{
-#'  \item{Q.orig}{The original Q-matrix that maybe contain some mis-specifications and need to be validated.}
-#'  \item{Q.sug}{The Q-matrix that suggested by certain validation method.}
-#'  \item{time.cost}{The time cost to finish the function.}
-#'  \item{process}{A matrix that contains the modification process of each question during each iteration. 
+#'  \item{\code{Q.orig}}{The original Q-matrix that maybe contain some mis-specifications and need to be validated.}
+#'  \item{\code{Q.sug}}{The Q-matrix that suggested by certain validation method.}
+#'  \item{\code{time.cost}}{The time cost to finish the function.}
+#'  \item{\code{process}}{A matrix that contains the modification process of each question during each iteration. 
 #'        Each row represents an iteration, and each column corresponds to the q-vector index of the respective 
 #'        question. The order of the indices is consistent with the row numbering in the matrix generated by 
 #'        the \code{\link[GDINA]{attributepattern}} function in the \code{GDINA} package. Only when 
 #'        \code{maxitr} > 1, the value is available.}
-#'  \item{iter}{The number of iteration. Only when \code{maxitr} > 1, the value is available.}
-#'  \item{priority}{An \code{I} × \code{K} matrix that contains the priority of every attribute for
+#'  \item{\code{iter}}{The number of iteration. Only when \code{maxitr} > 1, the value is available.}
+#'  \item{\code{priority}}{An \code{I} × \code{K} matrix that contains the priority of every attribute for
 #'                 each item. Only when the \code{search.method} is \code{"PAA"}, the value is available. See details.}
-#'  \item{Hull.fit}{A \code{list} containing all the information needed to plot the Hull plot, which is 
+#'  \item{\code{Hull.fit}}{A \code{list} containing all the information needed to plot the Hull plot, which is 
 #'                 available only when \code{method} = \code{"Hull"}.}
+#'  \item{\code{arguments}}{A list containing all input arguments}
 #' }
 #' 
 #' @author Haijiang Qin <Haijiang133@outlook.com>
@@ -420,13 +426,13 @@
 #'                          model = "GDINA", distribute = "horder")
 #'
 #' ## simulate random mis-specifications
-#' MQ <- sim.MQ(Q, 0.1)
+#' MQ <- sim.MQ(Q, 0.2)
 #'
 #' ## using MMLE/EM to fit CDM model first
 #' CDM.obj <- CDM(data$dat, MQ)
 #'
 #' ## using the fitted CDM.obj to avoid extra parameter estimation.
-#' Q.GDI.obj <- validation(data$dat, MQ, CDM.obj, method = "GDI")
+#' Q.GDI.obj <- validation(data$dat, MQ, CDM.obj=CDM.obj, method = "GDI")
 #' ## check QRR
 #' print(zQRR(Q, Q.GDI.obj$Q.sug))
 #'
@@ -452,47 +458,9 @@
 #'
 #'
 #'
+#'
 #' ################################################################
 #' #                           Example 2                          #
-#' #             The Wald method to validate Q-matrix             #
-#' ################################################################
-#' \donttest{
-#' set.seed(123)
-#'
-#' library(Qval)
-#'
-#' ## generate Q-matrix and data
-#' K <- 4
-#' I <- 20
-#' Q <- sim.Q(K, I)
-#' IQ <- list(
-#'   P0 = runif(I, 0.0, 0.2),
-#'   P1 = runif(I, 0.8, 1.0)
-#' )
-#' data <- sim.data(Q = Q, N = 500, IQ = IQ, model = "GDINA",
-#'                          distribute = "horder")
-#'
-#' ## simulate random mis-specifications
-#' MQ <- sim.MQ(Q, 0.1)
-#'
-#' ## using MMLE/EM to fit CDM first
-#' CDM.obj <- CDM(data$dat, MQ)
-#'
-#' ## using the fitted CDM.obj to avoid extra parameter estimation.
-#' Q.Wald.obj <- validation(data$dat, MQ, CDM.obj, method = "Wald")
-#'
-#'
-#' ## also can validate the Q-matrix directly
-#' Q.Wald.obj <- validation(data$dat, MQ, method = "Wald")
-#'
-#' ## check QRR
-#' print(zQRR(Q, Q.Wald.obj$Q.sug))
-#' }
-#'
-#'
-#'
-#' ################################################################
-#' #                           Example 3                          #
 #' #             The Hull method to validate Q-matrix             #
 #' ################################################################
 #' \donttest{
@@ -534,7 +502,7 @@
 #'
 #'
 #' ################################################################
-#' #                           Example 4                          #
+#' #                           Example 3                          #
 #' #             The MLR-B method to validate Q-matrix            #
 #' ################################################################
 #' \donttest{
@@ -561,6 +529,9 @@
 #'
 #' ## using the fitted CDM.obj to avoid extra parameter estimation.
 #' Q.MLR.obj <- validation(data$dat, MQ, CDM.obj, method = "MLR-B")
+#' 
+#' ## check QRR
+#' print(zQRR(Q, Q.MLR.obj$Q.sug))
 #'
 #'
 #' ## also can validate the Q-matrix directly
@@ -574,7 +545,7 @@
 #' @export
 #'
 validation <- function(Y, Q, 
-                       CDM.obj=NULL, par.method="EM", mono.constraint=TRUE, model="GDINA", 
+                       CDM.obj=NULL, par.method="EM", mono.constraint=FALSE, model="GDINA", 
                        method="GDI", search.method="PAA", iter.level="no", maxitr=1,
                        eps=0.95, alpha.level=NULL, criter = NULL, verbose = TRUE){
   
@@ -585,7 +556,6 @@ validation <- function(Y, Q,
                                 method, search.method, iter.level, maxitr,
                                 eps, alpha.level, criter, verbose)
   
-  
   Y <- check.obj$Y
   Q <- check.obj$Q
   CDM.obj <- check.obj$CDM.obj
@@ -603,23 +573,22 @@ validation <- function(Y, Q,
   
   time.cost <- system.time({
     if(method == "GDI"){
-      Qval.obj <- validation.GDI(Y, Q, CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
+      Qval.obj <- validation.GDI(Y=Y, Q=Q, CDM.obj=CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
                                  search.method=search.method, maxitr=maxitr, iter.level=iter.level, eps=eps, verbose=verbose)
     }else if(method == "Wald"){
       Qval.obj <- validation.Wald(Y=Y, Q=Q, CDM.obj=CDM.obj, mono.constraint=mono.constraint, search.method=search.method, 
                                   iter.level=iter.level, maxitr=maxitr, eps=eps, alpha.level=alpha.level, verbose=verbose)
     }else if(method == "Hull"){
-      Qval.obj <- validation.Hull(Y, Q, CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
+      Qval.obj <- validation.Hull(Y=Y, Q=Q, CDM.obj=CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
                                   search.method=search.method, maxitr=maxitr, iter.level=iter.level, criter=criter, verbose=verbose)
     }else if(method == "MLR-B"){
-      Qval.obj <- validation.MLR(Y, Q, CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
+      Qval.obj <- validation.MLR(Y=Y, Q=Q, CDM.obj=CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
                                  search.method=search.method, iter.level=iter.level, maxitr=maxitr, alpha.level=alpha.level, 
                                  verbose=verbose)
     }else if(method == "beta"){
-      Qval.obj <- validation.beta(Y, Q, CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
+      Qval.obj <- validation.beta(Y=Y, Q=Q, CDM.obj=CDM.obj, method=par.method, mono.constraint=mono.constraint, model=model, 
                                   search.method=search.method, maxitr=maxitr, iter.level=iter.level, criter=criter, verbose=verbose)
     }
-
   })
   
   res <- list(Q.orig = Qval.obj$Q.original, Q.sug = Qval.obj$Q.sug,
@@ -666,10 +635,15 @@ check.validation <- function(Y, Q,
   
   if (ncol(Q) == 1)
     stop("The number of attributes (K) must be greater than 1!")
-  
+
   if(!is.null(CDM.obj)){
-    if (!class(CDM.obj) %in% c("CDM", "NULL"))
-      stop("CDM.obj must be 'CDM' from CDM()!")
+    if (!class(CDM.obj) %in% c("CDM", "NULL")){
+      stop("CDM.obj must be 'CDM' from CDM()!") 
+    }
+    if(!is.null(CDM.obj$arguments$att.str)){
+      stop("CDM.obj contains a hierarchical structure; currently, 
+         there is no effective Q-matrix validation method that incorporates hierarchy.")
+    }
   }
   
   if (!par.method %in% c("EM", "BM"))
@@ -698,7 +672,7 @@ check.validation <- function(Y, Q,
     criter <- ifelse(is.null(criter),
                      ifelse(method == "Hull", 
                             "PVAF", ifelse(method == "beta", 
-                                   "AIC", NULL)), criter)
+                                           "AIC", NULL)), criter)
     if(method == "Hull"){
       if(!(criter %in% c("PVAF", "R2")))
         stop("criter must be one of c('PVAF', 'R2') when method = 'Hull'!")
