@@ -1,31 +1,19 @@
 #' @title Summary Methods for Various Objects
 #'
 #' @description
-#' Generate concise summary statistics for objects created by the Qval package.
+#' Generate concise summary statistics for objects created by the \code{Qval} package.
 #' The output is a named list tailored to the class of the input:
 #' \describe{
 #'   \item{\code{\link[Qval]{CDM}}}{contains the original call, dataset dimensions, model fit object, and attribute-pattern distribution.}
 #'   \item{\code{\link[Qval]{validation}}}{contains the original call, suggested Q-matrix, and original Q-matrix.}
 #'   \item{\code{\link[Qval]{sim.data}}}{contains the original call, dataset dimensions, and attribute-pattern distribution.}
 #'   \item{\code{\link[Qval]{fit}}}{contains the original call, relative fit indices and absolute fit indices.}
-#' }
-#'
-#' @details
-#' \describe{
-#'   \item{call}{A string capturing the original function invocation.}
-#'   \item{base}{A numeric vector \code{c(N, I, K)} giving the number of examinees (\eqn{N}), 
-#'               the number of items (\eqn{I}), and the number of attributes (\eqn{K}).}
-#'   \item{model.fit}{(CDM only) The fitted model object returned by \code{\link[Qval]{CDM}}.}
-#'   \item{patterns}{(CDM and sim.data) A data.frame of frequencies (\code{freq}) and proportions 
-#'                   (\code{prop}) of each attribute pattern.}
-#'   \item{Q.sug}{(validation only) Suggested Q-matrix from \code{\link[Qval]{validation}}.}
-#'   \item{Q.orig}{(validation only) Original Q-matrix provided by \code{\link[Qval]{sim.data}}.}
-#'   \item{fit.relative}{(fit only) Relative fit indices provided by \code{\link[Qval]{fit}}.}
-#'   \item{fit.absolute}{(fit only) Absolute fit indices provided by \code{\link[Qval]{fit}}.}
+#'   \item{\code{\link[Qval]{is.Qident}}}{contains the original call and results of whether the Q-matrix is identifiable.}
+#'   \item{\code{\link[Qval]{att.hierarchy}}}{contains the results of iterative attribute hierarchy exploration.}
 #' }
 #'
 #' @param object An object of class \code{\link[Qval]{CDM}}, \code{\link[Qval]{validation}}, 
-#'                \code{\link[Qval]{sim.data}}, or \code{\link[Qval]{fit}}.
+#'                \code{\link[Qval]{sim.data}}, \code{\link[Qval]{fit}}, or \code{\link[Qval]{is.Qident}}.
 #' @param ...   Currently unused. Additional arguments are ignored.
 #'
 #' @return A named list with class \code{summary.<class>} containing the components above.
@@ -71,11 +59,11 @@
 #' @name summary
 NULL
 
-#' @describeIn summary Summary method for CDM objects
+#' @describeIn summary Summary method for \code{CDM} objects
 #' @export
 summary.CDM <- function(object, ...) {
-  dat <- object$analysis.obj$Y
-  Q <- object$analysis.obj$Q
+  dat <- object$arguments$Y
+  Q <- object$arguments$Q
   catprob.parm <- object$analysis.obj$catprob.parm
   alpha <- object$alpha
   model.fit <- object$model.fit
@@ -102,7 +90,7 @@ summary.CDM <- function(object, ...) {
   return(out)
 }
 
-#' @describeIn summary Summary method for validation objects
+#' @describeIn summary Summary method for \code{validation} objects
 #' @export
 summary.validation <- function(object, ...){
   
@@ -120,7 +108,7 @@ summary.validation <- function(object, ...){
   return(out)
 }
 
-#' @describeIn summary Summary method for sim.data objects
+#' @describeIn summary Summary method for \code{sim.data} objects
 #' @export
 summary.sim.data <- function(object, ...) {
   
@@ -128,6 +116,7 @@ summary.sim.data <- function(object, ...) {
   Q <- object$Q
   catprob.parm <- object$catprob.parm
   alpha <- object$attribute
+  att.str <- object$att.str
   
   call <- paste(deparse(extract.CDM(object,"call"), width.cutoff = 30), sep = "\n", collapse = "\n")
   
@@ -143,14 +132,15 @@ summary.sim.data <- function(object, ...) {
   out <- list(
     call = call,
     base=base,
-    patterns = patterns
+    att.str=att.str, 
+    patterns = patterns 
   )
   
   class(out) <- "summary.sim.data"
   return(out)
 }
 
-#' @describeIn summary Summary method for fit objects
+#' @describeIn summary Summary method for \code{fit} objects
 #' @export
 summary.fit <- function(object, ...) {
   
@@ -171,4 +161,51 @@ summary.fit <- function(object, ...) {
   
   class(out) <- "summary.fit"
   return(out)
-}
\ No newline at end of file
+}
+
+#' @describeIn summary Summary method for \code{is.Qident} objects
+#' @export
+summary.is.Qident <- function(object, ...) {
+  
+  Q.orig <- object$Q.orig
+  Q.reconstructed <- object$Q.reconstructed
+  strictIdentifiability.obj <- object$strictIdentifiability.obj
+  genericIdentifiability.obj <- object$genericIdentifiability.obj
+  genericIdentifiability.DINA.obj <- object$genericIdentifiability.DINA.obj
+  
+  call <- paste(deparse(extract.fit(object,"call"), width.cutoff = 30), sep = "\n", collapse = "\n")
+  
+  out <- list(
+    call = call, 
+    Q.orig=Q.orig, 
+    strictIdentifiability.obj=strictIdentifiability.obj,
+    genericIdentifiability.obj=genericIdentifiability.obj, 
+    genericIdentifiability.DINA.obj=genericIdentifiability.DINA.obj
+  )
+  
+  class(out) <- "summary.is.Qident"
+  return(out)
+}
+
+#' @describeIn summary Summary method for \code{att.hierarchy} objects
+#' @export
+summary.att.hierarchy <- function(object, ...) {
+  
+  statistic <- object$statistic
+  pattern <- object$pattern
+  noSig <- object$noSig
+  isNonverge <- object$isNonverge
+  eps <- object$arguments$eps
+  
+  call <- paste(deparse(extract.fit(object,"call"), width.cutoff = 30), sep = "\n", collapse = "\n")
+  
+  out <- list(
+    call = call, eps=eps, 
+    statistic=statistic, pattern=pattern,
+    noSig=noSig, isNonverge=isNonverge
+  )
+  
+  class(out) <- "summary.att.hierarchy"
+  return(out)
+}
+
