@@ -2,12 +2,14 @@
 #' @description
 #' The \code{update} function provides a unified interface for refreshing or modifying
 #' existing analysis objects produced by the Qval package, including \code{\link[Qval]{CDM}},
-#' \code{\link[Qval]{validation}}, \code{\link[Qval]{sim.data}} and \code{\link[Qval]{fit}} classes. By passing additional arguments,
+#' \code{\link[Qval]{validation}}, \code{\link[Qval]{sim.data}}, \code{\link[Qval]{fit}}, 
+#' \code{\link[Qval]{is.Qident}}, \code{\link[Qval]{att.hierarchy}} classes. By passing additional arguments,
 #' users can rerun fitting or simulation routines without reconstructing the entire object
 #' from scratch.
-#'
+#' 
 #' @param x An object of class \code{\link[Qval]{CDM}}, \code{\link[Qval]{validation}}, 
-#'          and \code{\link[Qval]{sim.data}}.
+#'          \code{\link[Qval]{sim.data}}, \code{\link[Qval]{fit}}, \code{\link[Qval]{is.Qident}}, 
+#'          \code{\link[Qval]{att.hierarchy}}.
 #' @param ... Additional arguments specific to the method being updated:
 #'   \itemize{
 #'     \item For \code{CDM}: \code{Y}, \code{Q}, \code{model}, \code{method},
@@ -19,16 +21,18 @@
 #'     \item For \code{sim.data}: \code{Q}, \code{N}, \code{IQ}, \code{model},
 #'       \code{distribute}, \code{control}, \code{verbose}.
 #'     \item For \code{fit}: \code{Y}, \code{Q}, \code{model}.
+#'     ...
 #'   }
-#'
+#' 
 #' @return An updated object of the same class as \code{x}, reflecting any changes
 #'   in the supplied parameters.
-#'
+#' 
 #' @details
 #' The \code{update} methods internally extract the original call arguments
 #' from the input object, combine them with any new parameters provided in
 #' \code{...}, and re-invoke the corresponding constructor (\code{\link[Qval]{CDM}}, 
-#' \code{\link[Qval]{validation}}, \code{\link[Qval]{sim.data}} and \code{\link[Qval]{fit}}). 
+#' \code{\link[Qval]{validation}}, \code{\link[Qval]{sim.data}}, \code{\link[Qval]{fit}}, 
+#' \code{\link[Qval]{is.Qident}}), \code{\link[Qval]{att.hierarchy}}. 
 #' This approach ensures consistency and preserves all untouched settings from the original object.
 #' 
 #' @examples
@@ -88,7 +92,7 @@ update <- function(x, ...) {
   UseMethod("update")
 }
 
-#' @describeIn update Update method for CDM objects
+#' @describeIn update Update method for \code{CDM} objects
 #' @importFrom utils modifyList
 #' @export
 update.CDM <- function(x, ...) {
@@ -96,7 +100,8 @@ update.CDM <- function(x, ...) {
     Y = x$arguments$Y,
     Q = x$arguments$Q,
     model = x$arguments$model,
-    method = x$arguments$method,
+    method = x$arguments$method, 
+    att.str = x$arguments$att.str, 
     mono.constraint = x$arguments$mono.constraint,
     maxitr = x$arguments$maxitr,
     verbose = x$arguments$verbose
@@ -108,7 +113,8 @@ update.CDM <- function(x, ...) {
     Y = arguments.new$Y,
     Q = arguments.new$Q,
     model = arguments.new$model,
-    method = arguments.new$method,
+    method = arguments.new$method, 
+    att.str = arguments.new$att.str, 
     mono.constraint = arguments.new$mono.constraint,
     maxitr = arguments.new$maxitr,
     verbose = arguments.new$verbose
@@ -116,13 +122,13 @@ update.CDM <- function(x, ...) {
   return(CDM.updated)
 }
 
-#' @describeIn update Update method for validation objects
+#' @describeIn update Update method for \code{validation} objects
 #' @importFrom utils modifyList
 #' @export
 update.validation <- function(x, ...) {
   arguments.current <- list(
     Y = x$arguments$Y,
-    Q = x$arguments$Q,
+    Q = x$arguments$Q, 
     CDM.obj = x$arguments$CDM.obj,
     par.method = x$arguments$par.method,
     mono.constraint = x$arguments$mono.constraint,
@@ -141,7 +147,7 @@ update.validation <- function(x, ...) {
   
   validation.updated <- validation(
     Y = arguments.new$Y,
-    Q = arguments.new$Q,
+    Q = arguments.new$Q, 
     CDM.obj = arguments.new$CDM.obj,
     par.method = arguments.new$par.method,
     mono.constraint = arguments.new$mono.constraint,
@@ -158,14 +164,15 @@ update.validation <- function(x, ...) {
   return(validation.updated)
 }
 
-#' @describeIn update Update method for sim.data objects
+#' @describeIn update Update method for \code{sim.data} objects
 #' @importFrom utils modifyList
 #' @export
 update.sim.data <- function(x, ...) {
   arguments.current <- list(
     Q = x$arguments$Q,
     N = x$arguments$N,
-    IQ = x$arguments$IQ,
+    IQ = x$arguments$IQ, 
+    att.str = x$arguments$att.str, 
     model = x$arguments$model,
     distribute = x$arguments$distribute,
     control = x$arguments$control,
@@ -176,7 +183,8 @@ update.sim.data <- function(x, ...) {
   data.updated <- sim.data(
     Q = arguments.new$Q,
     N = arguments.new$N,
-    IQ = arguments.new$IQ,
+    IQ = arguments.new$IQ, 
+    att.str = arguments.new$att.str, 
     model = arguments.new$model,
     distribute = arguments.new$distribute,
     control = arguments.new$control,
@@ -185,21 +193,75 @@ update.sim.data <- function(x, ...) {
   return(data.updated)
 }
 
-#' @describeIn update Update method for fit objects
+#' @describeIn update Update method for \code{fit} objects
 #' @importFrom utils modifyList
 #' @export
 update.fit <- function(x, ...) {
   arguments.current <- list(
     Y = x$arguments$Y,
     Q = x$arguments$Q,
-    model = x$arguments$model
+    model = x$arguments$model, 
+    att.str = x$arguments$att.str, 
+    mono.constraint = x$arguments$mono.constraint,
+    maxitr = x$arguments$maxitr,
+    verbose = x$arguments$verbose
   )
   arguments.new <- modifyList(arguments.current, list(...))
   
   data.updated <- fit(
     Y = arguments.new$Y,
     Q = arguments.new$Q,
-    model = arguments.new$model
+    model = arguments.new$model, 
+    att.str = arguments.new$att.str, 
+    mono.constraint = arguments.new$mono.constraint,
+    maxitr = arguments.new$maxitr,
+    verbose = arguments.new$verbose
   )
   return(data.updated)
 }
+
+#' @describeIn update Update method for \code{is.Qident} objects
+#' @importFrom utils modifyList
+#' @export
+update.is.Qident <- function(x, ...) {
+  arguments.current <- list(
+    Q = x$arguments$Q,
+    verbose = x$arguments$verbose
+  )
+  arguments.new <- modifyList(arguments.current, list(...))
+  
+  data.updated <- is.Qident(
+    Q = arguments.new$Q,
+    verbose = arguments.new$verbose
+  )
+  return(data.updated)
+}
+
+#' @describeIn update Update method for \code{att.hierarchy} objects
+#' @importFrom utils modifyList
+#' @export
+update.att.hierarchy <- function(x, ...) {
+  arguments.current <- list(
+    Y = x$arguments$Y,
+    Q = x$arguments$Q,
+    model = x$arguments$model, 
+    mono.constraint = x$arguments$mono.constraint,
+    maxitr = x$arguments$maxitr, 
+    eps=x$arguments$eps, 
+    alpha.level=x$arguments$alpha.level, 
+    verbose = x$arguments$verbose
+  )
+  arguments.new <- modifyList(arguments.current, list(...))
+  
+  data.updated <- att.hierarchy(
+    Y = arguments.new$Y,
+    Q = arguments.new$Q,
+    model = arguments.new$model, 
+    mono.constraint = arguments.new$mono.constraint,
+    maxitr = arguments.new$maxitr, 
+    eps = arguments.new$eps, 
+    alpha.level = arguments.new$alpha.level, 
+    verbose = arguments.new$verbose
+  )
+  return(data.updated)
+}
\ No newline at end of file
