@@ -21,9 +21,9 @@
 #'
 #' @returns An object of class \code{htest} containing the following components:
 #' \describe{
-#'  \item{statistic}{The statistic of the Wald test.}
-#'  \item{parameter}{the degrees of freedom for the Wald-statistic.}
-#'  \item{p.value}{The p value}
+#'  \item{\code{statistic}}{The statistic of the Wald test.}
+#'  \item{\code{parameter}}{the degrees of freedom for the Wald-statistic.}
+#'  \item{\code{p.value}}{The p value}
 #' }
 #'
 #' @examples
@@ -67,14 +67,14 @@ Wald.test <- function(CDM.obj, q1, q2, i=1){
   }
   
   GDINA.obj <- CDM.obj$analysis.obj
-  Y <- GDINA.obj$Y
-  Q <- CDM.obj$analysis.obj$Q
+  Y <- CDM.obj$arguments$Y
+  Q <- CDM.obj$arguments$Q
   names.items <- rownames(Q)
   att.posi.i.k <- which(q2 > 0)
   att.posi.i <- which(q1 > 0)
   att.dif <- which(q1 != q2)
   
-  Qr <- GDINA.obj$Q
+  Qr <- Q
   Qr[i, seq_len(ncol(Qr))] <- 0
   Qr[i, c(att.posi.i, att.dif)] <- 1
   etas <- LC2LG(as.matrix(Qr))
@@ -86,10 +86,10 @@ Wald.test <- function(CDM.obj, q1, q2, i=1){
   index <- data.frame(Cat=rep(1:length(rowSums(Qr) ),2^rowSums(Qr)) )
   index$Column <- seq_len(length(index$Cat))
   
-  sco <- score(GDINA.obj, parm="prob") # a list with # of category elements
-  sco[[i]] <- score_pj(Xj = Y[, i],                   # a vector of item responses to item i
-                       parloc.j=etas[i, ,drop=FALSE],         # parameter locations for item i - H by 2^K matrix
-                       catprob.j=itemparj[i],        # a list with H elements giving the reduced catprob.parm for each nonzero category
+  sco <- score(GDINA.obj, parm="prob")
+  sco[[i]] <- score_pj(Xj = Y[, i],
+                       parloc.j=etas[i, ,drop=FALSE], 
+                       catprob.j=itemparj[i],
                        logpost=indlogPost(GDINA.obj))[[1]]
   
   ## v is the inversed information matrix
@@ -109,12 +109,7 @@ Wald.test <- function(CDM.obj, q1, q2, i=1){
   R.cov.Rt <- Rmatrix %*% cov %*% t(Rmatrix)  # R %*% cov %*% R^T
   R.P <- Rmatrix %*% P.Xi.alpha.reduced    # R %*% P.Xi.alpha.reduced
   
-  R.cov.Rt.det <- det(R.cov.Rt)
-  if (!is.na(R.cov.Rt.det)) {
-    R.cov.Rt.solved <- solve(R.cov.Rt)
-  }else{
-    R.cov.Rt.solved <- ginv(R.cov.Rt)
-  }
+  R.cov.Rt.solved <- MASS::ginv(R.cov.Rt)
   Wald.statistic <- t(R.P) %*% R.cov.Rt.solved %*%R.P
   
   parameter = nrow(Rmatrix)
