@@ -10,6 +10,9 @@
 #'           \code{P0} represents the probability of examinees who have not mastered any attributes  
 #'           (\eqn{[00...0]}) correctly answering the item, while \code{P1} represents the probability  
 #'           of examinees who have mastered all attributes (\eqn{[11...1]}) correctly answering the item.  
+#' @param att.str Specify attribute structures. \code{NULL}, by default, means there is no structure. Attribute structure 
+#'                needs be specified as a list - which will be internally handled by \code{\link[GDINA]{att.structure}} function. 
+#'                See examples. It can also be a matrix giving all permissible attribute profiles.
 #' @param model Type of model to be fitted; can be \code{"GDINA"}, \code{"LCDM"}, \code{"DINA"}, \code{"DINO"},
 #'              \code{"ACDM"}, \code{"LLM"}, or \code{"rRUM"}.
 #' @param distribute Attribute distributions; can be \code{"uniform"} for the uniform distribution,
@@ -27,6 +30,12 @@
 #'                   Default = 1.5 (Tu et al, 2022).
 #'     \item \code{b} The intercepts when \code{distribute = "horder"}. By default, select equally spaced
 #'                   values between -1.5 and 1.5 according to the number of attributes (Tu et al, 2022).
+#'     \item \code{alpha} Used to generate a structured parameter distribution with a hierarchical structure 
+#'                        when att.str is not NULL. This distribution is randomly drawn from a Dirichlet 
+#'                        distribution, where alpha denotes the parameters of the Dirichlet distribution, 
+#'                        and its length equals the number \code{L.str} of all valid attribute profiles 
+#'                        \eqn{\boldsymbol{\alpha}} under the hierarchical structure. Default by .
+#'                        \code{alpha = rep(1, L.str)}.
 #'  }
 #' @param verbose Logical indicating to print information or not. Default is \code{TRUE}
 #'
@@ -34,14 +43,14 @@
 #' An \code{sim.data} object initially gained by \code{\link[GDINA]{simGDINA}} function form \code{GDINA} package.
 #' Elements that can be extracted using method extract include:
 #' \describe{
-#'  \item{dat}{An \code{N} × \code{I} simulated item response matrix.}
-#'  \item{Q}{The Q-matrix.}
-#'  \item{attribute}{An \code{N} × \code{K} matrix for inviduals' attribute patterns.}
-#'  \item{catprob.parm}{A list of non-zero success probabilities for each attribute mastery pattern.}
-#'  \item{delta.parm}{A list of delta parameters.}
-#'  \item{higher.order.parm}{Higher-order parameters.}
-#'  \item{mvnorm.parm}{Multivariate normal distribution parameters.}
-#'  \item{LCprob.parm}{A matrix of success probabilities for each attribute mastery pattern.}
+#'  \item{\code{dat}}{An \code{N} × \code{I} simulated item response matrix.}
+#'  \item{\code{Q}}{The Q-matrix.}
+#'  \item{\code{attribute}}{An \code{N} × \code{K} matrix for inviduals' attribute patterns.}
+#'  \item{\code{catprob.parm}}{A list of non-zero success probabilities for each attribute mastery pattern.}
+#'  \item{\code{delta.parm}}{A list of delta parameters.}
+#'  \item{\code{higher.order.parm}}{Higher-order parameters.}
+#'  \item{\code{mvnorm.parm}}{Multivariate normal distribution parameters.}
+#'  \item{\code{LCprob.parm}}{A matrix of success probabilities for each attribute mastery pattern.}
 #' }
 #'
 #' @author Haijiang Qin <Haijiang133@outlook.com>
@@ -116,40 +125,89 @@
 #' print(data.obj$dat)
 #'
 #' @export
-#' @importFrom GDINA attributepattern
-#' @importFrom GDINA simGDINA
+#' @importFrom GDINA attributepattern att.structure simGDINA
 #' @importFrom stats runif qnorm rnorm sd
+#' @importFrom gtools  rdirichlet
 #' 
-#'
-
-sim.data <- function(Q=NULL, N=NULL, IQ=list(P0=NULL, P1=NULL),
-                     model="GDINA", distribute="uniform", control = NULL,
+sim.data <- function(Q=NULL, N=NULL, IQ=list(P0=NULL, P1=NULL), att.str=NULL, 
+                     model="GDINA", distribute="uniform", control = NULL, 
                      verbose = TRUE){
   
+  if(!is.null(att.str) & !class(att.str) %in% c("list", "matrix")){
+    stop("att.str must be NULL, a list or a matrix ! \n")
+  }else if(!is.null(att.str) & !model %in% c("GDINA", "DINA", "DINO") ){
+    stop("model must be one of 'GDINA', 'DINA' and 'DINO' when att.str is not NULL !\n")
+  }
+  
   simCall <- match.call()
-
+  
   if(is.null(Q))
-    Q <- sim.Q(5, 30)
+    Q <- sim.Q(5, 30, att.str)
   K <- ncol(Q)
   I <- nrow(Q)
   if(is.null(N))
     N <- 500
   if(is.null(IQ$P0))
-    IQ$P0 <- runif(I, 0, 0.3)
+    IQ$P0 <- runif(I, 0.0, 0.3)
   if(is.null(IQ$P1))
-    IQ$P1 <- runif(I, 0.7, 1)
+    IQ$P1 <- runif(I, 0.7, 1.0)
+  gs <- cbind(IQ$P0, 1 - IQ$P1)
+  
+  if(!is.null(att.str)){
+    distribute <- paste0(sapply(att.str, 
+                                function(x) return(paste0(paste0("A", x[1]), "->", paste0("A", x[2])))), 
+                         collapse = ", ")
+    wrapped.distribute <- strwrap(distribute, width = 80)
+    if(verbose)
+      cat("attribute structure: \n", paste(wrapped.distribute, collapse = "\n"), "\n")
+  }
+  
   if(verbose){
-    cat("distribute = ",distribute,"\n",
-        "model = ",model,"\n",
+    if(is.null(att.str)){
+      cat("distribute = ",distribute,"\n")
+    }
+    cat("model = ",model,"\n",
         "number of attributes: ", K, "\n",
         "number of items: ", I, "\n",
         "num of examinees: ", N, "\n",
         "average of P0 = ", round(mean(IQ$P0), 3), "\n",
         "average of P1 = ", round(mean(IQ$P1), 3), "\n")
   }
-
-  gs <- cbind(IQ$P0, 1 - IQ$P1)
-  if(distribute == "mvnorm") {
+  
+  if(!is.null(att.str)){
+    pattern <- attributepattern(K)
+    pattern.str <- att.structure(att.str, K)$att.str
+    L <- nrow(pattern)
+    
+    pattern.str.pattern <- sort(apply(pattern.str, 1, function(x) get_Pattern(x, pattern)))
+    att.prior <- rep(0, L)
+    
+    if(is.null(control$alpha)){
+      control$alpha <- rep(3, nrow(pattern.str))
+    }
+    att.prior[pattern.str.pattern] <- rdirichlet(n = 1, alpha = control$alpha)
+    
+    att.prior.names <- apply(pattern[pattern.str.pattern, ], 1, function(x) paste(x, collapse=""))
+    
+    if (verbose) {
+      alpha.str <- paste("Dirichlet's alpha =", paste(control$alpha, collapse = ", "))
+      cat(paste(strwrap(alpha.str, width = 80), collapse = "\n"), "\n")
+      names_vec <- att.prior.names
+      values_vec <- round(att.prior[pattern.str.pattern], 3)
+      max_name_width <- max(nchar(names_vec))
+      lines <- mapply(function(name, val) {
+        sprintf("%-*s : %.3f", max_name_width, name, val)
+      }, names_vec, values_vec, SIMPLIFY = TRUE)
+      cat("att.prior:\n")
+      cat(lines, sep = "\n")
+      cat("\n")
+    }
+    
+    data <- simGDINA(N, Q, gs.parm = gs, model = model, att.prior=att.prior, att.dist = "categorical", 
+                     gs.args = list(type = "random", mono.constraint = TRUE))
+  }
+  
+  if(all(distribute == "mvnorm")) {
     if(is.null(control$sigma)){
       sigma <- 0.5
     }else{
@@ -163,15 +221,15 @@ sim.data <- function(Q=NULL, N=NULL, IQ=list(P0=NULL, P1=NULL),
     if(verbose){
       cat("sigma =", round(sigma, 3), "\n", "cutoffs =", round(cutoffs, 3), "\n")
     }
-
+    
     vcov <- matrix(sigma,K,K)
     diag(vcov) <- 1
-    data <- simGDINA(N, Q, gs.parm = gs, model = model, att.dist = "mvnorm",
-                            gs.args = list(type = "random", mono.constraint = TRUE),
-                            mvnorm.parm=list(mean = rep(0,K), sigma = vcov, cutoffs = cutoffs))
+    data <- simGDINA(N, Q, gs.parm = gs, model = model, att.dist = "mvnorm", 
+                     gs.args = list(type = "random", mono.constraint = TRUE),
+                     mvnorm.parm=list(mean = rep(0,K), sigma = vcov, cutoffs = cutoffs))
   }
-
-  if(distribute == "horder") {
+  
+  if(all(distribute == "horder")) {
     if(is.null(control$theta)){
       theta <- rnorm(N, 0, 1)
     }else{
@@ -192,14 +250,15 @@ sim.data <- function(Q=NULL, N=NULL, IQ=list(P0=NULL, P1=NULL),
           "a = ", round(a, 3), "\n", "b = ", round(b, 3), "\n")
     }
 
-    data <- simGDINA(N, Q, gs.parm = gs, model = model, att.dist = "higher.order",
+    data <- simGDINA(N, Q, gs.parm = gs, model = model, att.dist = "higher.order", 
                      gs.args = list(type = "random", mono.constraint = TRUE),
                      higher.order.parm = list(theta = theta,lambda = data.frame(a=a, b=b)))
   }
 
-  if(distribute == "uniform")
-    data <- simGDINA(N, Q, gs.parm = gs, model = model,
-                            gs.args = list(type = "random", mono.constraint = TRUE))
+  if(all(distribute == "uniform")){
+    data <- simGDINA(N, Q, gs.parm = gs, model = model, 
+                     gs.args = list(type = "random", mono.constraint = TRUE))
+  }
   
    dat = data$dat
    Q = data$Q
@@ -216,16 +275,16 @@ sim.data <- function(Q=NULL, N=NULL, IQ=list(P0=NULL, P1=NULL),
               delta.parm=delta.parm, 
               higher.order.parm=higher.order.parm, 
               mvnorm.parm=mvnorm.parm, 
+              att.str=att.str, 
               LCprob.parm=LCprob.parm, 
               call = simCall, 
               arguments = list(
-                Q=Q, 
-                N=N, 
-                IQ=IQ,
+                Q=Q, N=N, IQ=IQ,
+                att.str=att.str, 
                 model=model, 
                 distribute=distribute, 
-                control = control,
-                verbose = verbose
+                control=control,
+                verbose=verbose
               ))
    
    class(out) <- "sim.data"
