@@ -8,7 +8,7 @@
 Rcpp::NumericVector P_GDINA(Rcpp::NumericVector& Qi, Rcpp::NumericVector& P_est, Rcpp::NumericMatrix& pattern, Rcpp::NumericVector& P_alpha) {
   
   size_t K = Qi.size();  // Use size_t for K
-  size_t L = static_cast<size_t>(pow(2, K));   // Use size_t for L
+  size_t L = pattern.nrow();   // Use size_t for L
   
   Rcpp::NumericVector P_Xj_alpha(L);
   
